/*!\file: numerics.h
 * \brief prototypes for numerics.h
 */ 

#ifndef _NUMERICS_H_
#define  _NUMERICS_H_

#include "./Verbosity.h"
#include "./GaussPoints.h"
#include "./isnan.h"

class Input;
class Parameters;

struct OptArgs;
struct OptPars;

double min(double a,double b);
double max(double a,double b);
int    min(int a,int b);
int    max(int a,int b);
double OptFunc(double scalar, OptArgs* optargs);
void   BrentSearch(double* psearch_scalar,double* pJ,OptPars* optpars,double (*f)(double,OptArgs*), OptArgs* optargs);
void   OptimalSearch(double* psearch_scalar,double* pJ,OptPars* optpars,double (*f)(double,OptArgs*), OptArgs* optargs);
void   cross(double* result,double* vector1,double* vector2);
void   IsInputConverged(double* peps, Input** new_inputs,Input** old_inputs,int num_inputs,int criterion_enum);
void   UnitConversion(double* values, int numvalues,int direction_enum, int type_enum);
double UnitConversion(double value, int direction_enum, int type_enum);
char*  OptionsFromAnalysis(Parameters* parameters,int analysis_type);
void   XZvectorsToCoordinateSystem(double* T,double* xzvectors);
#ifdef _HAVE_PETSC_
void   PetscOptionsFromAnalysis(Parameters* parameters,int analysis_type);
#endif

#endif //ifndef _NUMERICS_H_
